//
//  AFMethods.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/6/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//

#import "AFMethods.h"


@implementation AFMethods


+ (NSString*) fixBlankScore: (NSString*) score;
{
	/* When track provides a blank score, as when a wrestler has yet to score in their current match/period
	 * this method will replace the blank with a 0.  Otherwise, it will just return the score passed to the method
	 */
	
	NSString *fixedScore;
	
	if ([score isEqualToString: @""]) {
		fixedScore = [NSString stringWithString:@"0"];
	} else {
		fixedScore = score;
	}
	return fixedScore;
}


+ (NSString*) clockStringFromClockTimeInSeconds: (NSString*) clockTime clockRunning: (NSString*) clockRunning elapsedTimeInSeconds: (NSString*) elapsedTime;
{
	/* This function accepts the three values that provide information about the state of the clock.  Based on those inputs, it
	 * produces a string that shows a normal time format for display
	 */
	
	NSString *fixedClock;
	int clockTimeNumber = [clockTime intValue];
	int elapsedTimeNumber = [elapsedTime intValue];
	int seconds;
	int minutes;
	NSString *secondsString;
	
	if ([clockRunning isEqualToString:@"Y"] )
		clockTimeNumber = clockTimeNumber - elapsedTimeNumber;
	
	if (clockTimeNumber < 0)
		clockTimeNumber = 0;
	
	minutes = clockTimeNumber / 60;
	seconds = clockTimeNumber % 60;
	
	if (seconds < 10) 
		secondsString = [NSString stringWithFormat:@"0%d", seconds];
	else 
		secondsString = [NSString stringWithFormat:@"%d", seconds];

	
	fixedClock = [NSString stringWithFormat:@"%d:%@", minutes, secondsString];
	
	return fixedClock;
	
}


+ (NSArray*) cullXML: (NSString*) xml forTag: (NSString*) tag;
{
	/*
	 This accepts a tag and some proprietary track xml that uses [] rather than <>, and returns two values
	 1) the value between the specified open and close tag
	 2) the xml, modified excluding the first occurance of the specified tags and what was between them.
	 */
	
	
	/*
	 Build the tags that I will search for
	 */
	
	NSString *openTag = [NSString stringWithFormat: @"[%@]", tag];
	NSString *openTagAlternate = [NSString stringWithFormat: @"[%@ ", tag];
	NSString *closeTag = [NSString stringWithFormat: @"[/%@]", tag];
	
	/*
	 Find the locations of the possible first open tag
	 */
	
	NSRange openTagRange = [xml rangeOfString:openTag options:NSCaseInsensitiveSearch];
	NSRange	openTagAlternateRange = [xml rangeOfString:openTagAlternate options:NSCaseInsensitiveSearch];
	
	if (openTagAlternateRange.location >= 0 ) {
		if (openTagAlternateRange.location < openTagRange.location) {
			openTagRange = openTagAlternateRange;	
		}
	}
	
	/*
	 find the end tag, and build the correct strings
	 */
	
	
	NSRange openTagEndPosition;
	NSRange closeTagRange;
	NSMutableString *mutableXML = [NSMutableString stringWithString: xml];
	NSString *extractedText;
	NSString *remainingXML;
	
	if (openTagRange.location != NSNotFound ) {
		openTagEndPosition = [ xml rangeOfString:@"]" options:NSCaseInsensitiveSearch range:NSMakeRange(openTagRange.location, xml.length - openTagRange.location)];
		if (openTagEndPosition.location != NSNotFound && openTagEndPosition.location > openTagRange.location ) {
			closeTagRange = [xml rangeOfString:closeTag options:NSCaseInsensitiveSearch range:NSMakeRange(openTagEndPosition.location, xml.length - openTagEndPosition.location)];
			if (closeTagRange.location != NSNotFound && closeTagRange.location > openTagEndPosition.location ) {
				extractedText = [xml substringWithRange:NSMakeRange(openTagEndPosition.location + 1, closeTagRange.location - openTagEndPosition.location - 1)];
				[mutableXML deleteCharactersInRange:NSMakeRange(openTagRange.location, closeTagRange.location - openTagRange.location + closeTag.length)];
				remainingXML = mutableXML;
				return [NSArray arrayWithObjects: extractedText, remainingXML, nil];
			} 
		}
		
	}
	return [NSArray arrayWithObjects: nil];
	
}



+ (NSArray*) cullRealXML: (NSString*) xml forTag: (NSString*) tag;
{
	/*
	 This accepts a tag and some xml and returns two values
	 1) the value between the specified open and close tag
	 2) the xml, modified excluding the first occurance of the specified tags and what was between them.
	 */
	
	
	/*
	 Build the tags that I will search for
	 */
	
	NSString *openTag = [NSString stringWithFormat: @"<%@>", tag];
	NSString *openTagAlternate = [NSString stringWithFormat: @"<%@ ", tag];
	NSString *closeTag = [NSString stringWithFormat: @"</%@>", tag];
	
	/*
	 Find the locations of the possible first open tag
	 */
	
	NSRange openTagRange = [xml rangeOfString:openTag options:NSCaseInsensitiveSearch];
	NSRange	openTagAlternateRange = [xml rangeOfString:openTagAlternate options:NSCaseInsensitiveSearch];
	
	if (openTagAlternateRange.location >= 0 ) {
		if (openTagAlternateRange.location < openTagRange.location) {
			openTagRange = openTagAlternateRange;	
		}
	}
	
	/*
	 find the end tag, and build the correct strings
	 */
	
	
	NSRange openTagEndPosition;
	NSRange closeTagRange;
	NSMutableString *mutableXML = [NSMutableString stringWithString: xml];
	NSString *extractedText;
	NSString *remainingXML;
	
	if (openTagRange.location != NSNotFound ) {
		//NSLog(@"openTagRange.location: %d", openTagRange.location);
		openTagEndPosition = [ xml rangeOfString:@">" options:NSCaseInsensitiveSearch range:NSMakeRange(openTagRange.location, xml.length - openTagRange.location)];
		if (openTagEndPosition.location != NSNotFound && openTagEndPosition.location > openTagRange.location ) {
			//NSLog(@"openTageRange.location: %d, openTagEndPosition.location: %d", openTagRange.location, openTagEndPosition.location);
			closeTagRange = [xml rangeOfString:closeTag options:NSCaseInsensitiveSearch range:NSMakeRange(openTagEndPosition.location, xml.length - openTagEndPosition.location)];
			if (closeTagRange.location != NSNotFound && closeTagRange.location > openTagEndPosition.location ) {
				extractedText = [xml substringWithRange:NSMakeRange(openTagEndPosition.location + 1, closeTagRange.location - openTagEndPosition.location - 1)];
				[mutableXML deleteCharactersInRange:NSMakeRange(openTagRange.location, closeTagRange.location - openTagRange.location + closeTag.length)];
				remainingXML = mutableXML;
				return [NSArray arrayWithObjects: extractedText, remainingXML, nil];
			} 
		}
		
	}
	//[mutableXML release];
	return [NSArray arrayWithObjects: nil];
	
}

+ (NSArray*) tournamentTypeFromId: (NSString*) tournamentId;
{
	/*
	 * Things take a touranment ID, and returns the string associated with the tournament type in element 0, 
	 * and it returns the index for the type in element 1.
	 * If there's any goof up any where, it will return the values for 'open', even though that will probably never help.
	 * When track publishes the data for dual type events, this section will problably need to be changed
	 * When that happens, the routines that use this function may need to be changed as well.
	 */
	
	unichar tournamentTypeIndicator;
	NSArray *tournamentType;
	NSString *tournamentTypeString;
	NSString *tournamentTypeIndex;
	
	if ([tournamentId length] >= 1) {
		tournamentTypeIndicator = [tournamentId characterAtIndex:0];

		// figure out the tournament type
		switch (tournamentTypeIndicator) {
		
			case 'O' : tournamentTypeString = [NSString stringWithString:@"open"];
				tournamentTypeIndex = [NSString stringWithString:@"0"];
				break;
			case 'T' : tournamentTypeString = [NSString stringWithString:@"team"];
				tournamentTypeIndex = [NSString stringWithString:@"1"];
				break;
			case 'F' : tournamentTypeString = [NSString stringWithString:@"freestyle"];
				tournamentTypeIndex = [NSString stringWithString:@"2"];
				break;
			case 'P' : tournamentTypeString = [NSString stringWithString:@"predefined"];
				tournamentTypeIndex = [NSString stringWithString:@"3"];
				break;
			case 'D' : tournamentTypeString = [NSString stringWithString:@"dual"];
				tournamentTypeIndex = [NSString stringWithString:@"4"];
				break;
			default  : tournamentTypeString = [NSString stringWithString:@"open"];
				tournamentTypeIndex = [NSString stringWithString:@"0"];
				break;
		} 
	} else {
		tournamentTypeString = [NSString stringWithString:@"open"];
		tournamentTypeIndex = [NSString stringWithString:@"0"];
	}
	tournamentType	= [NSArray arrayWithObjects:tournamentTypeString, tournamentTypeIndex, nil];
	
	return tournamentType ;
}


+ (NSString*) baseURLFromServerName: (NSString*) serverName localServer: (BOOL) localServer;
{
	
	/*
	 * This method formulates the correct base URL for the Track data feeds
	 * it returns a string with the base url
	 */
	
	NSString *rootPath;
	NSString *portNumber;
	NSString *urlPrefix;
	
	if (localServer) {
		rootPath = [NSString stringWithString:@"examples/jsp/trackwrestling/"];
		portNumber = [NSString stringWithString:@"8080"];
	}
	else{				
		rootPath = [NSString stringWithString:@"tw/"];
		portNumber = [NSString stringWithString:@"80"];
	}
	urlPrefix = [NSString stringWithFormat:@"http://%@:%@/%@", serverName, portNumber, rootPath];
	
	return urlPrefix; 
}



+ (NSString*) stringFromURLString: (NSString*) url;

{
	NSURL *myURL = [NSURL URLWithString:url];
	NSURLRequest *urlRequest = [NSURLRequest requestWithURL:myURL
																							 cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																					 timeoutInterval:30];
	NSURLResponse *response = nil;
	NSError *error = nil;
	NSData *data = [NSURLConnection sendSynchronousRequest:urlRequest returningResponse:&response error:&error];
	NSString *string = [[NSString alloc] initWithData:data encoding: NSUTF8StringEncoding ];
	
	[string autorelease];
	NSLog(@"string: %@", string);
	return string;
	
}



@end
