//
//  AssignedBouts.m
//  Assigned Bouts
//
//  Created by Adam Fenn on 4/3/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "AssignedBouts.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"Assigned Bouts"
#define	kQCPlugIn_Description		@"Culls Track XML produced by GetAssignedMatches.jsp?tournamentId=... and returns a struture of assigned bout data for all defined mats."

@implementation AssignedBouts
@synthesize oldRefresh, response, responseData, newResults;

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputBaseUrl, inputTournamentId, inputRefresh, outputStructure;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
		if([key isEqualToString:@"inputBaseUrl"])
			return [NSDictionary dictionaryWithObjectsAndKeys:
			@"Base URL", QCPortAttributeNameKey,
			nil];
		if([key isEqualToString:@"inputTournamentId"])
			return [NSDictionary dictionaryWithObjectsAndKeys:
			@"Tournament ID", QCPortAttributeNameKey,
			nil];
		if([key isEqualToString:@"inputRefresh"])
			return [NSDictionary dictionaryWithObjectsAndKeys:
			@"Refresh", QCPortAttributeNameKey,
			nil];	
		if([key isEqualToString:@"outputStructure"])
			return [NSDictionary dictionaryWithObjectsAndKeys:
			@"Structure", QCPortAttributeNameKey,
			nil];	

	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];
	newResults = TRUE;
	
}

- (NSDictionary*) buildMatch: (NSString*) match;
{
	/*
	 * This method should accept a | delimited string of 40 fields describing a match, and return a dictionary
	 * with all of the fields nicely mapped into the dictionary with readable keys
	 */

	NSArray *returnedObjects;

	NSDictionary *currentMatch;
	NSMutableDictionary *presentedMatch = [[NSMutableDictionary alloc] init];
	NSArray *returnedDictionaryKeys;
	NSString *tournamentType;
	

	
																		 
	// determine tournament type, because the fields are different per type
	tournamentType = [[AFMethods tournamentTypeFromId:self.inputTournamentId] objectAtIndex:0];
	
	// split the | delimited values into an array
	returnedObjects = [match componentsSeparatedByString: @"|"];
	
	
	if ([tournamentType isEqualToString: @"open"] || [tournamentType isEqualToString: @"freestyle"]) {
		
		
		// set the correct field names for this tournament type;
		returnedDictionaryKeys = [NSArray arrayWithObjects:@"weightId", @"uniqueBackgroundBoutNumber", @"prelimInd", @"matchId",
															@"boutNumber", @"mat", @"division", @"weight", @"boutType", @"w1FirstName", @"w1LastName", @"w1Team",
															@"w2FirstName", @"w2LastName", @"w2Team", @"numberSchemeId", @"w1TWId", @"w2TWId",
															@"w1Seed", @"w1Dob", @"w1Club", @"w1Hometown", @"w1State", @"w1Grade", @"w1Phone", @"w1Address", @"w1Zip",
															@"w1Color", @"w2Seed", @"w2Dob", @"w2Club", @"w2Hometown", @"w2State", @"w2Grade", @"w2Phone", @"w2Address",
															@"w2Zip", @"w2Color", @"uniqueBoutKey", @"matOrderNumber", nil ];
		
		// make sure we got at least 40 fields returned
		if (returnedObjects.count >= 40 ) {
			// chop off anything after the 40 fields we know about
			returnedObjects = [returnedObjects subarrayWithRange:NSMakeRange(0, 40)];
			// make  reading the array easy  - this will crash if it's greater than 40.
			currentMatch = [NSDictionary dictionaryWithObjects:returnedObjects forKeys:returnedDictionaryKeys];
			
			// clean up the fields into something that is more usable (clock and wrestler data)
			if ([[returnedObjects objectAtIndex:27] isEqualToString: @"r"] || [[returnedObjects objectAtIndex:27] isEqualToString: @""] ) {
				// red all of w1 values, and blue all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"redSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Dob"] forKey:@"redDob"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Club"] forKey:@"redClub"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Hometown"] forKey:@"redHometown"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1State"] forKey:@"redState"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Phone"] forKey:@"redPhone"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Address"] forKey:@"redAddress"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Zip"] forKey:@"redZip"];
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"blueSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Dob"] forKey:@"blueDob"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Club"] forKey:@"blueClub"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Hometown"] forKey:@"blueHometown"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2State"] forKey:@"blueState"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"blueGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Phone"] forKey:@"bluePhone"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Address"] forKey:@"blueAddress"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Zip"] forKey:@"blueZip"];
				
			} else {
				// blue all of w1 values and red all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"redSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Dob"] forKey:@"redDob"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Club"] forKey:@"redClub"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Hometown"] forKey:@"redHometown"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2State"] forKey:@"redState"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Phone"] forKey:@"redPhone"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Address"] forKey:@"redAddress"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Zip"] forKey:@"redZip"];
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"blueSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Dob"] forKey:@"blueDob"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Club"] forKey:@"blueClub"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Hometown"] forKey:@"blueHometown"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1State"] forKey:@"blueState"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"blueGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Phone"] forKey:@"bluePhone"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Address"] forKey:@"blueAddress"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Zip"] forKey:@"blueZip"];
				
			}
			
			// handle the neutral fields
			[presentedMatch setValue: [currentMatch valueForKey:@"weightId"] forKey:@"weightId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"uniqueBackgroundBoutNumber"] forKey:@"uniqueBackgroundBoutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"prelimInd"] forKey:@"prelimInd"];
			[presentedMatch setValue: [currentMatch valueForKey:@"matchId"] forKey:@"matchId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber"] forKey:@"boutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"mat"] forKey:@"mat"];
			[presentedMatch setValue: [currentMatch valueForKey:@"division"] forKey:@"division"];
			[presentedMatch setValue: [currentMatch valueForKey:@"weight"] forKey:@"weight"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutType"] forKey:@"boutType"];
			[presentedMatch setValue: [currentMatch valueForKey:@"numberSchemeId"] forKey:@"numberSchemeId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w2Color"] forKey:@"w2Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Color"] forKey:@"w1Color"];	
			[presentedMatch setValue: [currentMatch valueForKey:@"uniqueBoutKey"] forKey:@"uniqueBoutKey"];
			[presentedMatch setValue: [currentMatch valueForKey:@"matOrderNumber"] forKey:@"matOrderNumber"];
			
			
		}
	} else if ([tournamentType isEqualToString: @"predefined"]) {
		
		
		// set the correct field names for this tournament type;
		returnedDictionaryKeys = [NSArray arrayWithObjects:@"weightId", @"uniqueBackgroundBoutNumber", @"prelimInd", @"matchId",
															@"boutNumber", @"mat", @"weight", @"boutType", @"w1FirstName", @"w1LastName", @"w1Team", @"w1TeamAbbr", @"w1Record", @"w1Grade",
															@"w2FirstName", @"w2LastName", @"w2Team", @"w2TeamAbbr", @"w2Record", @"w2Grade", @"numberSchemeId", @"w1TWId", @"w2TWId",
															@"tmOrderNumber", @"w1Seed", @"w1Info",
															@"w1Color", @"w2Seed", @"w2Info", @"w2Color", @"uniqueBoutKey", nil ];
		
		// make sure we got at least 31 fields returned
		if (returnedObjects.count >= 31 ) {
			// chop off anything after the 31 fields we know about
			returnedObjects = [returnedObjects subarrayWithRange:NSMakeRange(0, 31)];
			// make  reading the array easy  - this will crash if it's greater than 31.
			currentMatch = [NSDictionary dictionaryWithObjects:returnedObjects forKeys:returnedDictionaryKeys];
			
			// clean up the fields into something that is more usable (clock and wrestler data)
			if ([[returnedObjects objectAtIndex:26] isEqualToString: @"r"] || [[returnedObjects objectAtIndex:26] isEqualToString: @""] ) {
				// red all of w1 values, and blue all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamAbbr"] forKey:@"redTeamAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"redRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"redSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Info"] forKey:@"redInfo"];

				
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamAbbr"] forKey:@"blueTeamAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"blueRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"blueGrade"];				
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"blueSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Info"] forKey:@"blueInfo"];

				
			} else {
				// blue all of w1 values and red all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamAbbr"] forKey:@"redTeamAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"redRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"redSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Info"] forKey:@"redInfo"];
				
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamAbbr"] forKey:@"blueTeamAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"blueRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"blueGrade"];				
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"blueSeed"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Info"] forKey:@"blueInfo"];
				
			}
			
			// handle the neutral fields
			[presentedMatch setValue: [currentMatch valueForKey:@"weightId"] forKey:@"weightId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"uniqueBackgroundBoutNumber"] forKey:@"uniqueBackgroundBoutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"prelimInd"] forKey:@"prelimInd"];
			[presentedMatch setValue: [currentMatch valueForKey:@"matchId"] forKey:@"matchId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber"] forKey:@"boutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"mat"] forKey:@"mat"];
			[presentedMatch setValue: [currentMatch valueForKey:@"weight"] forKey:@"weight"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutType"] forKey:@"boutType"];
			[presentedMatch setValue: [currentMatch valueForKey:@"numberSchemeId"] forKey:@"numberSchemeId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w2Color"] forKey:@"w2Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Color"] forKey:@"w1Color"];	
			[presentedMatch setValue: [currentMatch valueForKey:@"uniqueBoutKey"] forKey:@"uniqueBoutKey"];
			[presentedMatch setValue: [currentMatch valueForKey:@"tmOrderNumber"] forKey:@"tmOrderNumber"];
			
			
		}
		
	}
	[presentedMatch autorelease];

	return presentedMatch;

}

@end

@implementation AssignedBouts (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	
		
	// start the URL load via async
	if (self.inputRefresh && self.inputRefresh != self.oldRefresh && self.inputTournamentId.length > 3) {
		// build the url to load
		NSString *tournamentId = [self.inputTournamentId substringWithRange:NSMakeRange(3,self.inputTournamentId.length - 3 )   ];
		NSString *urlString = [NSString stringWithFormat:@"%@GetAssignedMatches.jsp?tournamentId=%@", self.inputBaseUrl, tournamentId];
		
		NSURL *myURL = [NSURL URLWithString:urlString];
		
		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																			cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																	timeoutInterval:2];
		
		[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
		self.newResults = FALSE;
	}
	self.oldRefresh = self.inputRefresh;
	
	// start processing the results, if they exist
	if (self.newResults) {
		
		NSString *urlResult;
		NSArray *matsXML;
		NSArray *matXML;
		NSArray *matName;
		NSArray *schemeId;
		NSArray *match;
		NSMutableDictionary *mats = [[NSMutableDictionary alloc] init];
		NSMutableDictionary *mat = [[NSMutableDictionary alloc] init];
		NSDictionary *currentMatch;
		int i = 0;
		
		urlResult = [NSString stringWithFormat:@"%@", self.response];
		
		
		
		matsXML = [AFMethods cullXML: urlResult forTag: @"mats"];
		
		if ( matsXML.count > 0) {
			// load up the first mat for processing
			matXML = [AFMethods cullXML: [matsXML objectAtIndex:0] forTag: @"mat"];
			
			while (matXML.count > 0) { 
				
				// grab the mat name
				matName = [AFMethods cullXML:[matXML objectAtIndex:0] forTag:@"matName"];
				if (matName.count > 0) {
					[mat setObject:[matName objectAtIndex:0] forKey:@"matName"];
				}
				// get the schemeId
				schemeId = [AFMethods cullXML:[matXML objectAtIndex:0] forTag:@"schemeId"];
				if (schemeId.count > 0) {
					[mat setObject:[schemeId objectAtIndex:0] forKey:@"schemeId"];
				}
				
				// grab the first match on this mat
				match = [AFMethods cullXML: [matXML objectAtIndex:0] forTag: @"match"];
				if (match.count > 0) {
					currentMatch = [self buildMatch:[match objectAtIndex:0]];
					// put the processed match in the dictionary
					[mat setObject:currentMatch forKey:@"activeMatch"];
					
					
					// grab the second match
					match = [AFMethods cullXML: [match objectAtIndex:1] forTag: @"match"];
					if (match.count > 0) {
						currentMatch = [self buildMatch:[match objectAtIndex:0]];
						// put the processed match in the dictionary
						[mat setObject:currentMatch forKey:@"onDeck"];
						
						
						// grab the thrid match
						match = [AFMethods cullXML: [match objectAtIndex:1] forTag: @"match"];
						if (match.count > 0) {
							currentMatch = [self buildMatch:[match objectAtIndex:0]];
							// put the third match in the dictionary
							[mat setObject:currentMatch forKey:@"inHole"];
							
							
						} 
						
					} 
					
				} 
				
				// load up the next mat for processing
				matXML = [AFMethods cullXML: [matXML objectAtIndex:1] forTag: @"mat"];
				
				// put the mat's match list in the output dictionary
				[mats setObject:[NSDictionary dictionaryWithDictionary:mat] forKey: [NSString stringWithFormat: @"%d", i++ ]];
				
				// wipe out the data in the elements of mat details
				[mat removeAllObjects];
				
				
			}	
			
			
		}
		self.outputStructure = mats; 
		
		[mats release];
		[mat release];
	}
	return YES;
	
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}


@end


