//
//  AssignedBoutsTickerStringPlugIn.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/11/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//

#import "AssignedBoutsTickerStringPlugIn.h"


/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "CreateTickerStringsPlugIn.h"

#define	kQCPlugIn_Name				@"Assigned Bouts Ticker String"
#define	kQCPlugIn_Description		@"Accepts the as,signed bouts structure full of data to be transformed into ticker information, and spits out two strings.  The first string is for on deck information, the second string is for in the hole information."

@implementation AssignedBoutsTickerStringPlugIn

/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputAssignmentData, inputSeparator, outputOnDeckString, outputInHoleString;


+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputAssignmentData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Assignment Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputSeparator"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Separator", QCPortAttributeNameKey,
						nil];
	
	if([key isEqualToString:@"outputOnDeckString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"On Desk String", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputInHoleString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"In Hole String", QCPortAttributeNameKey,
						nil];	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	
	[super dealloc];
}




@end

@implementation AssignedboutsTickerStringPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */
	
	
	NSString *resultsString = [NSString	stringWithString:@""];
	NSString *thisResult;
	NSString *key;
	NSDictionary *thisDictionary;
	int i;
	NSDictionary *dictionary = [NSDictionary dictionaryWithDictionary:self.inputResultsData];
	NSString *separator = [NSString stringWithString:self.inputSeparator];
	
	
	
	for (i=0; i < dictionary.count; i++) {
		key = [NSString stringWithFormat:@"%i", i];
		thisDictionary = [dictionary objectForKey:key];
		thisResult = [NSString stringWithFormat:@"%@ %@ over %@ %@ %@ %@", [thisDictionary objectForKey:@"wFName"], [thisDictionary objectForKey:@"wLName"],
									[thisDictionary objectForKey:@"lFName"], [thisDictionary objectForKey:@"lLName"],
									[thisDictionary objectForKey:@"winTypeAbbr"], [thisDictionary objectForKey:@"scoreSummary"]];
		resultsString = [resultsString stringByAppendingFormat:@"%@%@", thisResult, separator];
	}
	
	self.outputResultsString = resultsString;
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end