//
//  BoutResultsPlugIn.m
//  Bout Results
//
//  Created by Adam Fenn on 4/6/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "BoutResultsPlugIn.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"Bout Results"
#define	kQCPlugIn_Description		@"Culls Track XML produced by GetRecentMatchResults.jsp?tournamentId=...&limit=... and returns a structure populated with the results information.  It will pull the number of bouts specified with the limit URL parameter."

@implementation BoutResultsPlugIn
@synthesize oldRefresh, response, responseData, newResults;
/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputBaseUrl, inputLimit, inputTournamentId, inputRefresh, outputStructure;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	if([key isEqualToString:@"inputBaseUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Base URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputLimit"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Limit", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputTournamentId"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Tournament ID", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];
	
	if([key isEqualToString:@"outputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Structure", QCPortAttributeNameKey,
						nil];	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];
	newResults = TRUE;	
}


@end

@implementation BoutResultsPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/


	// start the URL load via async
	if (self.inputRefresh && self.inputRefresh != self.oldRefresh && self.inputTournamentId.length > 3) {
		// build the url to load
		NSString *tournamentId = [self.inputTournamentId substringWithRange:NSMakeRange(3,self.inputTournamentId.length - 3 )   ];
		NSString *urlString = [NSString stringWithFormat:@"%@GetRecentMatchResults.jsp?tournamentId=%@&limit=%i", self.inputBaseUrl, tournamentId, (int)self.inputLimit];
		
		NSURL *myURL = [NSURL URLWithString:urlString];

		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																													 cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																											 timeoutInterval:2];

		[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
		self.newResults = FALSE;
	}
	self.oldRefresh = self.inputRefresh;
	if (self.newResults) {
		NSString *urlResult;
		NSArray *matchesXML;
		NSArray *match;
		NSDictionary *currentResult;
		NSArray *dictionaryKeys = [NSArray arrayWithObjects:@"boutType", @"boutNo", @"division", @"weight", @"winTypeAbbr", @"winTypePhrase", @"scoreSummary", @"wFName", @"wLName", @"wTeam", @"lFName", @"lLName", @"lTeam", @"wTWId", @"lTWId", @"weightId", nil];	
		NSMutableDictionary *results = [[NSMutableDictionary alloc] init];
		NSArray *returnedObjects;
		NSMutableArray *currentResultObjects = [[NSMutableArray alloc] initWithObjects:[NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], [NSNull null], nil];
		
		int i = 0;
		
		urlResult = [NSString stringWithFormat:@"%@", self.response];
		
		matchesXML = [AFMethods cullXML: urlResult forTag: @"matches"];
		
		if ( matchesXML.count > 0) {
			// load up the first match for processing
			match = [AFMethods cullXML: [matchesXML objectAtIndex:0] forTag: @"match"];
			
			while (match.count > 0)  {
				
				// split the | delimited values into an array
				returnedObjects = [[match objectAtIndex:0] componentsSeparatedByString: @"|"];
				
				// make sure we got 16 at least 16 fields returned
				if (returnedObjects.count >= 16 ) {
					
					// copy only 16 returned Objects inot mutuable which keeps the limit at 16
					[currentResultObjects replaceObjectsInRange:NSMakeRange(0,16)  withObjectsFromArray: returnedObjects range:NSMakeRange(0, 16)];				
					
					// populate the current result
					currentResult = [NSDictionary dictionaryWithObjects:currentResultObjects forKeys:dictionaryKeys];
					// put it in the output structure
					[results setObject:currentResult forKey: [NSString stringWithFormat: @"%d", i++ ]];
					
					//load the next result into match in prep for restarting the loop
				}
				match = [AFMethods cullXML: [match objectAtIndex:1] forTag: @"match"];
			} 
		}

		self.outputStructure = results; 
		
		[results release];
		[currentResultObjects release];

	}
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
