//
//  CreateTickerStringsPlugIn.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/11/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "CreateTickerStringsPlugIn.h"

#define	kQCPlugIn_Name				@"Create Ticker Strings"
#define	kQCPlugIn_Description		@"Accepts structures full of data to be transformed into ticker information, and spits out a single string for each feed."

@implementation CreateTickerStringsPlugIn

/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputResultsData, inputAssignmentData, inputTeamScoresData, inputHeadlinesData, inputSeparator, outputResultsString, outputOnDeckString, outputInHoleString, outputTeamScoresString, outputHeadlinesString;


+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputResultsData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Results Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputAssignmentData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Assignment Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputTeamScoresData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Team Scores Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputHeadlinesData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Headlines Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputSeparator"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Separator", QCPortAttributeNameKey,
						nil];

	if([key isEqualToString:@"outputResultsString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Results String", QCPortAttributeNameKey,
						nil];	

	if([key isEqualToString:@"outputOnDeckString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"On Deck String", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputInHoleString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"In Hole String", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTeamScoresString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Team Scores String", QCPortAttributeNameKey,
						nil];		
	if([key isEqualToString:@"outputHeadlinesString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Headlines String", QCPortAttributeNameKey,
						nil];	
	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	
	[super dealloc];
}




@end

@implementation CreateTickerStringsPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */
	
	
	NSString *resultsString = [NSString	stringWithString:@""];
	NSString *headlinesString = [NSString	stringWithString:@""];
	NSString *onDeckString = [NSString	stringWithString:@""];
	NSString *inHoleString = [NSString	stringWithString:@""];
	NSString *teamScoresString = [NSString	stringWithString:@""];
	NSString *thisResult;
	NSDictionary *thisMatDictionary;
	NSDictionary *thisDictionary;
	int i;
	int lastTeamStanding = 0;
	int teamStanding = 0;
	NSString *lastTeamScore = [NSString stringWithString:@"" ];
	NSString *separator = [NSString stringWithString:self.inputSeparator];
	NSString *key;
	NSArray *structure;
	NSDictionary *dictionary = [NSDictionary dictionaryWithDictionary:self.inputResultsData];
	
	for (i=0; i < dictionary.count; i++) {
		key = [NSString stringWithFormat:@"%i", i];
		thisDictionary = [dictionary objectForKey:key];
		if ( [thisDictionary isKindOfClass:[NSDictionary class]]) {
			thisResult = [NSString stringWithFormat:@"%@ %@ over %@ %@ - %@", [thisDictionary objectForKey:@"wFName"], [thisDictionary objectForKey:@"wLName"],
										[thisDictionary objectForKey:@"lFName"], [thisDictionary objectForKey:@"lLName"],
										[thisDictionary objectForKey:@"scoreSummary"]];
			resultsString = [resultsString stringByAppendingFormat:@"%@%@", thisResult, separator];
		}
	}
	
	
	dictionary = [NSDictionary dictionaryWithDictionary:self.inputAssignmentData];
	for (i=0; i < dictionary.count; i++) {
		key = [NSString stringWithFormat:@"%i", i];
		thisMatDictionary = [dictionary objectForKey:key];
		if ([thisMatDictionary isKindOfClass:[NSDictionary class]] ) {
			
			// handle on deck
			thisDictionary = [thisMatDictionary objectForKey:@"onDeck"];
			if ([thisDictionary isKindOfClass:[NSDictionary class]]) {
				if ([[thisDictionary objectForKey:@"boutNumber"] length ] > 0 ) {
					thisResult = [NSString stringWithFormat:@"%@ (Bout %@) %@ %@ vs. %@ %@", [thisDictionary objectForKey:@"mat"], [thisDictionary objectForKey:@"boutNumber"],
												[thisDictionary objectForKey:@"redFirstName"], [thisDictionary objectForKey:@"redLastName"],
												[thisDictionary objectForKey:@"blueFirstName"], [thisDictionary objectForKey:@"blueLastName"]];
					
					onDeckString = [onDeckString stringByAppendingFormat:@"%@%@", thisResult, separator];
				}
			}

			// handle in the hole
			thisDictionary = [thisMatDictionary objectForKey:@"inHole"];			
			if ([thisDictionary isKindOfClass:[NSDictionary class]]) {
				if ([[thisDictionary objectForKey:@"boutNumber"] length ] > 0 ) {
					thisResult = [NSString stringWithFormat:@"%@ (Bout %@) %@ %@ vs. %@ %@", [thisDictionary objectForKey:@"mat"], [thisDictionary objectForKey:@"boutNumber"],
												[thisDictionary objectForKey:@"redFirstName"], [thisDictionary objectForKey:@"redLastName"],
												[thisDictionary objectForKey:@"blueFirstName"], [thisDictionary objectForKey:@"blueLastName"]];
					
					inHoleString = [inHoleString stringByAppendingFormat:@"%@%@", thisResult, separator];
				}
			}
		}
	}
	
	dictionary = [NSDictionary dictionaryWithDictionary:self.inputTeamScoresData];

	for (i=0; i < dictionary.count; i++) {
		key = [NSString stringWithFormat:@"%i", i];

		thisDictionary = [dictionary objectForKey:key];
		if ([thisDictionary isKindOfClass:[NSDictionary class]] ) {

			// handle ties appropriately
			if ([[thisDictionary objectForKey:@"teamPoints"] isEqualToString:lastTeamScore]) {
				teamStanding = lastTeamStanding;
			}
			else {
				teamStanding = i+1;
			}
			
			thisResult = [NSString stringWithFormat:@"%i. %@ - %@", teamStanding, [thisDictionary objectForKey:@"teamName"],
										[thisDictionary objectForKey:@"teamPoints"]];
			
			teamScoresString = [teamScoresString stringByAppendingFormat:@"%@%@", thisResult, separator];
			lastTeamStanding = teamStanding;
			lastTeamScore = [thisDictionary objectForKey:@"teamPoints"];
		}
	}
	
	structure = [NSArray arrayWithArray:self.inputHeadlinesData];
	
	for (i=0; i < structure.count; i++) {

		thisResult = [structure objectAtIndex:i];
		headlinesString = [headlinesString stringByAppendingFormat:@"%@%@", thisResult, separator];
	}
	
	
	self.outputHeadlinesString = headlinesString;
	self.outputOnDeckString = onDeckString;
	self.outputInHoleString = inHoleString;
	self.outputResultsString = resultsString;
	self.outputTeamScoresString = teamScoresString;
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end
