//
//  CurrentMatchDataPlugIn.m
//  CurrentMatchData
//
//  Created by Adam Fenn on 4/6/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "CurrentMatchDataPlugIn.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"Current Match Data"
#define	kQCPlugIn_Description		@"Culls information from Track XML produced by GetCurrentMatchData.jsp?tournamentId=... and returns a structure of structures filled with bout information on the current bout on each mat."

@implementation CurrentMatchDataPlugIn
@synthesize oldRefresh, response, responseData, newResults;

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/
@dynamic inputBaseUrl, inputTournamentId, inputRefresh, outputStructure;


+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	
	if([key isEqualToString:@"inputBaseUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Base URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputTournamentId"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"TournamentId", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Structure", QCPortAttributeNameKey,
						nil];	

	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];
	newResults = TRUE;
	
}

- (NSDictionary*) buildMatch: (NSString*) match;
{
	/*
	 * This method should accept a | delimited string of 40 fields describing a match, and return a dictionary
	 * with all of the fields nicely mapped into the dictionary with readable keys
	 */
	
	
	NSArray *returnedObjects;
	NSDictionary *currentMatch;
	NSMutableDictionary *presentedMatch = [[NSMutableDictionary alloc] init];
	NSArray *returnedDictionaryKeys; 
	NSString *tournamentType;
	NSString *clockString;
	
	// determine tournament type, because the fields are different per type
	tournamentType = [[AFMethods tournamentTypeFromId:self.inputTournamentId] objectAtIndex:0];
		
	// split the | delimited values into an array
	returnedObjects = [match componentsSeparatedByString: @"|"];
	
	// make sure we got at least 32 fields returned
	if (returnedObjects.count >= 32 ) {
		// chop off anything after the 32 fields we know about
		returnedObjects = [returnedObjects subarrayWithRange:NSMakeRange(0, 32)];
		
		if ([tournamentType isEqualToString: @"open"] || [tournamentType isEqualToString: @"freestyle"]) {
			
			// assign the correct names to the fields per this type
			returnedDictionaryKeys = [NSArray arrayWithObjects:@"isMatchOver", @"boutNumberAssigned", @"mat", @"weightClass",
																@"boutType", @"w1FirstName", @"w1LastName", @"w1Team",
																@"w2FirstName", @"w2LastName", @"w2Team", @"clockTimeInSeconds",
																@"clockRunning", @"elapsedTimeInSeconds", @"w1MatchScore", @"w2MatchScore",
																@"w1Color", @"w2Color", @"periodName", @"matchId", 
																@"w1TWId", @"w2TWId", @"groupId", @"boutNumber", 
																@"prelimInd", @"w1Position", @"division", @"periodScore1",
																@"periodScore2", @"w1Periods", @"w2Periods", @"tcStyle", nil];
			
			// make  reading the array easy  - this will crash if it's greater than 32.
			currentMatch = [NSDictionary dictionaryWithObjects:returnedObjects forKeys:returnedDictionaryKeys];
			
			// clean up the fields into something that is more usable (clock and wrestler data)
			if ([[returnedObjects objectAtIndex:18] isEqualToString: @"r"] || [[returnedObjects objectAtIndex:18] isEqualToString: @""] ) {
				// red all of w1 values, and blue all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore1"]] forKey:@"redPeriodScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Periods"] forKey:@"redPeriods"];
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore2"]] forKey:@"bluePeriodScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Periods"] forKey:@"bluePeriods"];
				
			} else {
				// blue all of w1 values and red all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"redTWId"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore2"]] forKey:@"redPeriodScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Periods"] forKey:@"redPeriods"];
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"blueTWId"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore1"]] forKey:@"bluePeriodScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Periods"] forKey:@"bluePeriods"];
				
			}
			
			// handle the neutral fields
			[presentedMatch setValue: [currentMatch valueForKey:@"isMatchOver"] forKey:@"isMatchOver"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumberAssigned"] forKey:@"boutNumberAssigned"];
			[presentedMatch setValue: [currentMatch valueForKey:@"mat"] forKey:@"mat"];
			[presentedMatch setValue: [currentMatch valueForKey:@"weightClass"] forKey:@"weightClass"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutType"] forKey:@"boutType"];
			[presentedMatch setValue: [currentMatch valueForKey:@"clockRunning"] forKey:@"clockRunning"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Color"] forKey:@"w1Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w2Color"] forKey:@"w2Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"periodName"] forKey:@"periodName"];
			[presentedMatch setValue: [currentMatch valueForKey:@"matchId"] forKey:@"matchId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"groupId"] forKey:@"groupId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber"] forKey:@"boutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"prelimInd"] forKey:@"prelimInd"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Position"] forKey:@"w1Position"];
			[presentedMatch setValue: [currentMatch valueForKey:@"division"] forKey:@"division"];
			[presentedMatch setValue: [currentMatch valueForKey:@"tcStyle"] forKey:@"tcStyle"];
			
			// handle the clock
			clockString = [AFMethods clockStringFromClockTimeInSeconds: [currentMatch valueForKey:@"clockTimeInSeconds"] 
																										clockRunning: [currentMatch valueForKey:@"clockRunning"]
																						elapsedTimeInSeconds: [currentMatch valueForKey:@"elapsedTimeInSeconds"]];
			
			[presentedMatch setValue: clockString forKey:@"clockString"];
		} else if ([tournamentType isEqualToString: @"predefined"]) {
			
			// assign the correct names to the fields per this type
			returnedDictionaryKeys = [NSArray arrayWithObjects:@"isMatchOver", @"boutNumberAssigned", @"mat", @"weightClass",
																@"boutType", @"w1FirstName", @"w1LastName", @"w1Team", @"w1Grade",
																@"w2FirstName", @"w2LastName", @"w2Team", @"w2Grade", @"clockTimeInSeconds",
																@"clockRunning", @"elapsedTimeInSeconds", @"w1MatchScore", @"w2MatchScore",
																@"w1Color", @"w2Color", @"periodName", @"matchId", @"w1Record", @"w1GradeAbbr", @"w2Record", @"w2GradeAbbr",
																@"w1TWId", @"w2TWId", @"groupId", @"boutNumber", 
																@"prelimInd", @"w1Position", nil];
			
			// make  reading the array easy  - this will crash if it's greater than 32.
			currentMatch = [NSDictionary dictionaryWithObjects:returnedObjects forKeys:returnedDictionaryKeys];
			
			// clean up the fields into something that is more usable (clock and wrestler data)
			if ([[returnedObjects objectAtIndex:18] isEqualToString: @"r"] || [[returnedObjects objectAtIndex:18] isEqualToString: @""] ) {
				// red all of w1 values, and blue all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"redRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1GradeAbbr"] forKey:@"redGradeAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"redTWId"];

				
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"blueGrade"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"blueRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2GradeAbbr"] forKey:@"blueGradeAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"blueTWId"];

			} else {
				// blue all of w1 values and red all of w2
				[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"redFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"redLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"redTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"redGrade"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"redMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"redRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2GradeAbbr"] forKey:@"redGradeAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"redTWId"];
				
				
				[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"blueFirstName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"blueLastName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"blueTeamName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"blueGrade"];
				[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"blueMatchScore"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"blueRecord"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1GradeAbbr"] forKey:@"blueGradeAbbr"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"blueTWId"];
				
			}
			
			// handle the neutral fields
			[presentedMatch setValue: [currentMatch valueForKey:@"isMatchOver"] forKey:@"isMatchOver"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumberAssigned"] forKey:@"boutNumberAssigned"];
			[presentedMatch setValue: [currentMatch valueForKey:@"mat"] forKey:@"mat"];
			[presentedMatch setValue: [currentMatch valueForKey:@"weightClass"] forKey:@"weightClass"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutType"] forKey:@"boutType"];
			[presentedMatch setValue: [currentMatch valueForKey:@"clockRunning"] forKey:@"clockRunning"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Color"] forKey:@"w1Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w2Color"] forKey:@"w2Color"];
			[presentedMatch setValue: [currentMatch valueForKey:@"periodName"] forKey:@"periodName"];
			[presentedMatch setValue: [currentMatch valueForKey:@"matchId"] forKey:@"matchId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"groupId"] forKey:@"groupId"];
			[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber"] forKey:@"boutNumber"];
			[presentedMatch setValue: [currentMatch valueForKey:@"prelimInd"] forKey:@"prelimInd"];
			[presentedMatch setValue: [currentMatch valueForKey:@"w1Position"] forKey:@"w1Position"];
			
			// handle the clock
			clockString = [AFMethods clockStringFromClockTimeInSeconds: [currentMatch valueForKey:@"clockTimeInSeconds"] 
																										clockRunning: [currentMatch valueForKey:@"clockRunning"]
																						elapsedTimeInSeconds: [currentMatch valueForKey:@"elapsedTimeInSeconds"]];
			
			[presentedMatch setValue: clockString forKey:@"clockString"];
		}
		
		
	}
	[presentedMatch autorelease];
	return presentedMatch;
	
}



@end

@implementation CurrentMatchDataPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	
	// start the URL load via async
	if (self.inputRefresh && self.inputRefresh != self.oldRefresh && self.inputTournamentId.length > 3) {
		// build the url to load
		NSString *tournamentId = [self.inputTournamentId substringWithRange:NSMakeRange(3,self.inputTournamentId.length - 3 )   ];
		NSString *urlString = [NSString stringWithFormat:@"%@GetCurrentMatchData.jsp?tournamentId=%@", self.inputBaseUrl, tournamentId];
		
		NSURL *myURL = [NSURL URLWithString:urlString];
		
		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																			cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																	timeoutInterval:2];
		
		[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
		self.newResults = FALSE;
	}
	self.oldRefresh = self.inputRefresh;
	if (newResults) {
		
		NSString *urlResult;
		NSArray *matchesXML;
		NSArray *match;
		NSDictionary *currentMatch;
		NSMutableDictionary *matches = [[NSMutableDictionary alloc] init];
		
		
		
		urlResult = [NSString stringWithFormat:@"%@", self.response];
		
		matchesXML = [AFMethods cullXML: urlResult forTag: @"matches"];
		
		if ( matchesXML.count > 0) {
			// load up the first match for processing
			match = [AFMethods cullXML: [matchesXML objectAtIndex:0] forTag: @"match"];
			
			while (match.count > 0)  {
				
				// load up a match
				currentMatch = [self buildMatch:[match objectAtIndex:0]];
				
				// put the match in the array, indexed by matName?
				[matches setObject:currentMatch forKey:[currentMatch objectForKey:@"mat"]];
				
				//load the next team into match in prep for restarting the loop
				match = [AFMethods cullXML: [match objectAtIndex:1] forTag: @"match"];
				
			} 
		}
		self.outputStructure = matches; 
		
		[matches release];
	}
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
