//
//  LiveBoutDataPlugIn.m
//  LiveBoutData
//
//  Created by Adam Fenn on 4/7/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "LiveBoutDataPlugIn.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"Live Bout Data"
#define	kQCPlugIn_Description		@"This plugin pulls the available fields from track, just as they would appear on the clock.  It massages the data so that the clock is usable, and so that the wrestlers are identifieable via their colors.  It presents the bout information in a keyed structure."

@implementation LiveBoutDataPlugIn
@synthesize oldRefresh, response, responseData, newResults;

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputBaseUrl, inputRefresh, outputBoutData, inputBoutKey;


+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	
	if([key isEqualToString:@"inputBaseUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Base URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputBoutData"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Bout Data", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputBoutKey"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Bout Key", QCPortAttributeNameKey,
						nil];
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];	
	newResults = TRUE;
	
}

@end

@implementation LiveBoutDataPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	

	if (self.inputBaseUrl.length != 0 && self.inputBoutKey.length > 1 ) {
		
		
		if (self.inputRefresh && self.inputRefresh != self.oldRefresh ) {
			// build the url to load
			NSString *urlString = [NSString stringWithFormat:@"%@ClockInfo.jsp?boutKey=%@&includeFields=0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60", self.inputBaseUrl, self.inputBoutKey]; 
			
			NSURL *myURL = [NSURL URLWithString:urlString];
			
			NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																														 cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																												 timeoutInterval:.4];
			
			[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
			self.newResults = FALSE;
		}
		self.oldRefresh = self.inputRefresh;
		
		if (self.newResults) {
			
			NSString *urlResult;
			NSArray *returnedObjects;
			NSString *clockString;
			NSDictionary *currentMatch;
			NSMutableDictionary *presentedMatch = [[NSMutableDictionary alloc] init];
			NSArray *returnedDictionaryKeys = [NSArray arrayWithObjects:@"boutNumber", @"mat", @"division", @"weightClass", @"boutType",
																				 @"w1FirstName", @"w1LastName", @"w1Team", @"w1Seed", @"w1ActualWeight", @"w1Dob", @"w1Club",
																				 @"w1Hometown", @"w1State", @"w1Grade", @"w2FirstName", @"w2LastName", @"w2Team", @"w2Seed",
																				 @"w2ActualWeight", @"w2Dob", @"w2Club", @"w2Hometown", @"w2State", @"w2Grade", @"w1ExtraField",
																				 @"w2ExtraField", @"w1Phone", @"w2Phone", @"w1Address", @"w2Address", @"w1Zip", @"w2Zip", 
																				 @"clockTimeInSeconds", @"clockRunning", @"elapsedTimeInSeconds", @"w1MatchScore",
																				 @"w2MatchScore", @"isMatchOver", @"w1Color", @"w2Color", @"periodName", @"periodScore1",
																				 @"periodScore2", @"matchId", @"w1Periods", @"w2Periods", @"w1Record", @"w1GradeAbbr", 
																				 @"w2Record", @"w2GradeAbbr", @"w1TWId", @"w2TWId", @"groupId", @"boutNumber2", @"prelimInd",
																				 @"w1Position", @"w1TeamScore", @"w2TeamScore", @"w1TeamId", @"w2TeamId", nil];
			
			
			
			urlResult = [NSString stringWithFormat:@"%@", self.response];
			
			urlResult = [urlResult stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			urlResult = [urlResult stringByTrimmingCharactersInSet:[NSCharacterSet newlineCharacterSet]];
			
			// split the ~ delimited values into an array
			returnedObjects = [urlResult componentsSeparatedByString: @"~"];
			
			// make sure we got at least 32 fields returned
			if (returnedObjects.count >= 61 ) {
				
				// make sure we only work with the first 61 objects
				returnedObjects = [returnedObjects subarrayWithRange:NSMakeRange(0, 61)];
				
				// make  reading the array easy
				currentMatch = [NSDictionary dictionaryWithObjects:returnedObjects forKeys:returnedDictionaryKeys];
				
				// clean up the fields into something that is more usable (clock and wrestler data)
				if ([[returnedObjects objectAtIndex:39] isEqualToString: @"r"] || [[returnedObjects objectAtIndex:39] isEqualToString: @""] ) {
					// map fields for when w1 is red
					[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"redFirstName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"redLastName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"redTeam"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"redSeed"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1ActualWeight"] forKey:@"redActualWeight"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Dob"] forKey:@"redDob"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Club"] forKey:@"redClub"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Hometown"] forKey:@"redHometown"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1State"] forKey:@"redState"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"redGrade"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1ExtraField"] forKey:@"redExtraField"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Phone"] forKey:@"redPhone"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Address"] forKey:@"redAddress"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Zip"] forKey:@"redZip"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"redMatchScore"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore1"]] forKey:@"redPeriodScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Periods"] forKey:@"redPeriods"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"redRecord"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1GradeAbbr"] forKey:@"redGradeAbbr"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"redTWId"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamScore"] forKey:@"redTeamScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamId"] forKey:@"redTeamId"];
					
					[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"blueFirstName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"blueLastName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"blueTeam"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"blueSeed"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2ActualWeight"] forKey:@"blueActualWeight"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Dob"] forKey:@"blueDob"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Club"] forKey:@"blueClub"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Hometown"] forKey:@"blueHometown"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2State"] forKey:@"blueState"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"blueGrade"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2ExtraField"] forKey:@"blueExtraField"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Phone"] forKey:@"bluePhone"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Address"] forKey:@"blueAddress"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Zip"] forKey:@"blueZip"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"blueMatchScore"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore2"]] forKey:@"bluePeriodScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Periods"] forKey:@"bluePeriods"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"blueRecord"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2GradeAbbr"] forKey:@"blueGradeAbbr"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"blueTWId"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamScore"] forKey:@"blueTeamScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamId"] forKey:@"blueTeamId"];
					
					
				} else {
					// map fields for when w1 is blue
					[presentedMatch setValue: [currentMatch valueForKey:@"w2FirstName"] forKey:@"redFirstName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2LastName"] forKey:@"redLastName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Team"] forKey:@"redTeam"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Seed"] forKey:@"redSeed"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2ActualWeight"] forKey:@"redActualWeight"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Dob"] forKey:@"redDob"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Club"] forKey:@"redClub"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Hometown"] forKey:@"redHometown"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2State"] forKey:@"redState"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Grade"] forKey:@"redGrade"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2ExtraField"] forKey:@"redExtraField"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Phone"] forKey:@"redPhone"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Address"] forKey:@"redAddress"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Zip"] forKey:@"redZip"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w2MatchScore"]] forKey:@"redMatchScore"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore2"]] forKey:@"redPeriodScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Periods"] forKey:@"redPeriods"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2Record"] forKey:@"redRecord"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2GradeAbbr"] forKey:@"redGradeAbbr"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TWId"] forKey:@"redTWId"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamScore"] forKey:@"redTeamScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w2TeamId"] forKey:@"redTeamId"];
					
					[presentedMatch setValue: [currentMatch valueForKey:@"w1FirstName"] forKey:@"blueFirstName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1LastName"] forKey:@"blueLastName"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Team"] forKey:@"blueTeam"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Seed"] forKey:@"blueSeed"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1ActualWeight"] forKey:@"blueActualWeight"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Dob"] forKey:@"blueDob"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Club"] forKey:@"blueClub"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Hometown"] forKey:@"blueHometown"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1State"] forKey:@"blueState"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Grade"] forKey:@"blueGrade"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1ExtraField"] forKey:@"blueExtraField"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Phone"] forKey:@"bluePhone"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Address"] forKey:@"blueAddress"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Zip"] forKey:@"blueZip"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"w1MatchScore"]] forKey:@"blueMatchScore"];
					[presentedMatch setValue: [AFMethods fixBlankScore:[currentMatch valueForKey:@"periodScore1"]] forKey:@"bluePeriodScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Periods"] forKey:@"bluePeriods"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1Record"] forKey:@"blueRecord"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1GradeAbbr"] forKey:@"blueGradeAbbr"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TWId"] forKey:@"blueTWId"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamScore"] forKey:@"blueTeamScore"];
					[presentedMatch setValue: [currentMatch valueForKey:@"w1TeamId"] forKey:@"blueTeamId"];
					
				}
				// handle all of the neutral fields
				
				[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber"] forKey:@"boutNumber"];
				[presentedMatch setValue: [currentMatch valueForKey:@"mat"] forKey:@"mat"];
				[presentedMatch setValue: [currentMatch valueForKey:@"division"] forKey:@"division"];
				[presentedMatch setValue: [currentMatch valueForKey:@"weightClass"] forKey:@"weightClass"];
				[presentedMatch setValue: [currentMatch valueForKey:@"boutType"] forKey:@"boutType"];
				[presentedMatch setValue: [currentMatch valueForKey:@"isMatchOver"] forKey:@"isMatchOver"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Color"] forKey:@"w1Color"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w2Color"] forKey:@"w2Color"];
				[presentedMatch setValue: [currentMatch valueForKey:@"periodName"] forKey:@"periodName"];
				[presentedMatch setValue: [currentMatch valueForKey:@"matchId"] forKey:@"matchId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"groupId"] forKey:@"groupId"];
				[presentedMatch setValue: [currentMatch valueForKey:@"boutNumber2"] forKey:@"boutNumber2"];
				[presentedMatch setValue: [currentMatch valueForKey:@"prelimInd"] forKey:@"prelimInd"];
				[presentedMatch setValue: [currentMatch valueForKey:@"w1Position"] forKey:@"w1Position"];
				[presentedMatch setValue: [currentMatch valueForKey:@"clockRunning"] forKey:@"clockRunning"];
				
				
				// handle the clock
				clockString = [AFMethods clockStringFromClockTimeInSeconds: [currentMatch valueForKey:@"clockTimeInSeconds"] 
																											clockRunning: [currentMatch valueForKey:@"clockRunning"]
																							elapsedTimeInSeconds: [currentMatch valueForKey:@"elapsedTimeInSeconds"]];
				
				[presentedMatch setValue: clockString forKey:@"clockString"];
				
				self.outputBoutData = [NSDictionary dictionaryWithDictionary:presentedMatch];
				[presentedMatch release];
				
			}
		}
		
	}
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
