//
//  ReadRSSFeedPlugIn.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/13/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "ReadRSSFeedPlugIn.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"RSS Titles to Structure"
#define	kQCPlugIn_Description		@"Culls XML produced by an RSS feed and returns a structure populated with the specified number of titles."

@implementation ReadRSSFeedPlugIn
@synthesize oldRefresh, response, responseData, newResults;
/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputUrl, inputRefresh, inputLimit, outputStructure;


+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"RSS URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputLimit"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Limit", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Structure", QCPortAttributeNameKey,
						nil];	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSUTF8StringEncoding];
	newResults= TRUE;
	[responseData release];
	[connection release];
}




@end

@implementation ReadRSSFeedPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */
	
	
	// start the URL load via async
	if (self.inputRefresh && self.inputRefresh != self.oldRefresh ) {
		NSURL *myURL = [NSURL URLWithString:self.inputUrl];
		
		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																			cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																	timeoutInterval:2];
		
		[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
		self.newResults= FALSE;
	}
	
	self.oldRefresh = self.inputRefresh;
	
	if (self.newResults) {
		
		NSString *urlResult;
		NSArray *rssFeedXML;
		NSArray *currentItemXML;
		NSArray *currentTitle;
		NSMutableArray *titles = [[NSMutableArray alloc] init];
		int i = 0;

		urlResult = [NSString stringWithFormat:@"%@", self.response];
		
		// start processign the data
		rssFeedXML = [AFMethods cullRealXML: urlResult forTag: @"rss"];
		
		if (rssFeedXML.count > 0 ) {
			currentItemXML = [AFMethods cullRealXML: [rssFeedXML objectAtIndex:0] forTag: @"item"];
			
			while (currentItemXML.count > 0 && i < self.inputLimit)  {
				
				currentTitle = [AFMethods cullRealXML: [currentItemXML objectAtIndex:0] forTag:@"title"];
				if (currentTitle.count > 0) {
					// add this title to the dictionary
					//[titles setValue: [currentTitle objectAtIndex:0] forKey: [NSString stringWithFormat:@"%i", i++] ];
					[titles addObject:[currentTitle objectAtIndex:0]];
					i++;
				}
				
				//load the next team into match in prep for restarting the loop
				currentItemXML = [AFMethods cullRealXML: [currentItemXML objectAtIndex:1] forTag: @"item"];
			} 
		}
		self.outputStructure = titles; 
		
		[titles release];
	}
	return YES;
		
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end

