//
//  StructureKeysPlugIn.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/10/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//


/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "StructureKeysPlugIn.h"

#define	kQCPlugIn_Name				@"Structure Keys"
#define	kQCPlugIn_Description		@"Produces an indexed structure of they keys of the provided structure"

@implementation StructureKeysPlugIn

/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputStructure, outputKeys;


+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Structure", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputKeys"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Keys", QCPortAttributeNameKey,
						nil];	

	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) b
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	
	[super dealloc];
}

NSComparisonResult numericCompare( id str1, id str2, void *context )
{
	return [str1 compare:str2 options:NSNumericSearch|NSCaseInsensitiveSearch];
}

@end

@implementation StructureKeysPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */

	NSDictionary *inputDictionary;
	NSMutableArray *array = [[NSMutableArray alloc] init];
	NSArray *sorted;
	
	inputDictionary = [NSDictionary dictionaryWithDictionary:self.inputStructure];
	
	
	for (id key in inputDictionary) {	
		[array addObject:key];
	}
	sorted = [array sortedArrayUsingFunction:numericCompare context:nil];


	
	self.outputKeys = sorted;
	
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end
