//
//  TeamScoresPlugIn.m
//  TeamScores
//
//  Created by Adam Fenn on 4/6/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "TeamScoresPlugIn.h"
#import "AFMethods.h"

#define	kQCPlugIn_Name				@"Team Scores"
#define	kQCPlugIn_Description		@"Culls Track XML produced by GetTeamScores.jsp?tournamentId=... and returns a structure populated with team score information"

@implementation TeamScoresPlugIn
@synthesize oldRefresh, response, responseData, newResults;
/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputBaseUrl, inputTournamentId, inputRefresh, outputStructure;


+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	if([key isEqualToString:@"inputBaseUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Base URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputTournamentId"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"TournamentId", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Structure", QCPortAttributeNameKey,
						nil];	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];
	newResults = TRUE;
	
}




@end

@implementation TeamScoresPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	
	// start the URL load via async
	if (self.inputRefresh && self.inputRefresh != self.oldRefresh && self.inputTournamentId.length > 3) {
		// build the url to load
		NSString *tournamentId = [self.inputTournamentId substringWithRange:NSMakeRange(3,self.inputTournamentId.length - 3 )   ];
		NSString *urlString = [NSString stringWithFormat:@"%@GetTeamScores.jsp?tournamentId=%@", self.inputBaseUrl, tournamentId];
		
		NSURL *myURL = [NSURL URLWithString:urlString];
		
		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																			cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																	timeoutInterval:2];
		
		[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
		self.newResults = FALSE;
	}
	self.oldRefresh = self.inputRefresh;
	
	if (self.newResults) {
		
		
		NSString *urlResult;
		NSArray *teamsXML;
		NSArray *team;
		NSDictionary *currentTeam;
		NSArray *dictionaryKeys = [NSArray arrayWithObjects:@"teamId", @"teamName", @"teamPoints", nil];	
		NSMutableDictionary *teams = [[NSMutableDictionary alloc] init];
		NSArray *returnedObjects;
		NSMutableArray *currentTeamObjects = [[NSMutableArray alloc] initWithObjects:[NSNull null], [NSNull null], [NSNull null], nil];
		int i = 0;
		
		urlResult = [NSString stringWithFormat:@"%@", self.response];
		
		
		teamsXML = [AFMethods cullXML: urlResult forTag: @"teams"];
		
		if ( teamsXML.count > 0) {
			// load up the first team for processing
			team = [AFMethods cullXML: [teamsXML objectAtIndex:0] forTag: @"team"];
			
			while (team.count > 0)  {
				
				// split the | delimited values into an array
				returnedObjects = [[team objectAtIndex:0] componentsSeparatedByString: @"|"];
				
				// make sure we got 3 at least 3 fields returned
				if (returnedObjects.count >= 3 ) {
					
					// copy only 3 returned Objects into mutuable which keeps the limit at 3
					[currentTeamObjects replaceObjectsInRange:NSMakeRange(0,3)  withObjectsFromArray: returnedObjects range:NSMakeRange(0,3)];				
					
					// populate the current team
					currentTeam = [NSDictionary dictionaryWithObjects:currentTeamObjects forKeys:dictionaryKeys];
					// put it in the output structure
					[teams setObject:currentTeam forKey: [NSString stringWithFormat: @"%d", i++ ]];
				}
				
				//load the next team into match in prep for restarting the loop
				team = [AFMethods cullXML: [team objectAtIndex:1] forTag: @"team"];
			} 
		}
		self.outputStructure = teams; 
		
		[teams release];
		[currentTeamObjects release];
		
		
	}
	
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
