//
//  TickerElementPlugIn.m
//  WrestlingTools
//
//  Created by Adam Fenn on 4/26/11.
//  Copyright 2011 Adam Fenn. All rights reserved.
//


/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "TickerElementPlugIn.h"

#define	kQCPlugIn_Name				@"Ticker Element"
#define	kQCPlugIn_Description		@"Accepts a number of values and produces the formatting, positioning and timing values for a ticker element display."

@implementation TickerElementPlugIn
@synthesize startTime, tickerScrolling, titleSlidingIn, titleScrollingOut, done;

/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputParentSaysStart, inputTickerTitle, inputSpeed, inputEnable, 
	inputOnlyOneEnabled, inputParentDone, inputTickerWidth, inputPatchTime, outputTickerXPosition, 
	outputBorderWidth, outputTitleBackgroundWidth, outputBorderXPosition, outputTitleXPosition, 
	outputTitleYPosition, outputBackgroundHeight, outputStartChild, outputDone, outputEnabled;


+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputParentSaysStart"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Start", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputTickerTitle"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Ticker Title", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputSpeed"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Speed", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputEnable"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Enable", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputOnlyOneEnabled"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Only One Enabled", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputParentDone"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Parent Done", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputPatchTime"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Patch Time", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputTickerWidth"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Ticker Width", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTickerXPosition"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Ticker X Position", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputBorderWidth"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Border Width", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTitleBackgroundWidth"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Title Background Width", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputBorderXPosition"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Border X Position", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTitleXPosition"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Title X Position", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTitleYPosition"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Title Y Position", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputBackgroundHeight"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Background Height", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputStartChild"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Start Child", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputDone"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Done", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputEnabled"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Enabled", QCPortAttributeNameKey,
						nil];	

	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeIdle;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	
	[super dealloc];
}




@end

@implementation TickerElementPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */
	//NSLog(@"starting up");
	
	self.outputEnabled = self.inputEnable;
	
	if (self.inputEnable) {
		
		double halfWidth;
		double runningTime;
		double offLeft;
		double titleXPosition;
		
		// only do the calculations when I'm enabled
		//NSLog(@"I'm enabled");
		
		
		// determine when to start and if I'm running;
		
		if (self.inputParentDone) {
			startTime = self.inputPatchTime;
			tickerScrolling = TRUE;
			titleSlidingIn = TRUE;
			//NSLog(@"parent is done and startTime is %f", startTime);
			

		}
		//NSLog(@"tickerScrolling is %i", tickerScrolling);
		
		if (tickerScrolling) {
			
			/////////////////////
			// slide the title through
			/////////////////////
			
			//NSLog(@"doing my calculations for x");
			
			halfWidth = self.inputTickerWidth /2 ;
			// calculate my x 
			runningTime = self.inputPatchTime - startTime;
			//NSLog(@"after runningTime: %f", runningTime);
			double offRight = 1 + halfWidth;
			//NSLog(@"after offRight: %f", offRight);
			offLeft = -1 - halfWidth;
			//NSLog(@"after offLeft: %f", offLeft);
			//NSLog(@"speed is %f", self.inputSpeed);
						
			double tickerXPosition = offRight - runningTime * self.inputSpeed;
			//NSLog(@"after tickerXPosition");
			//NSLog(@"checking ot see if I am done, x is %f", tickerXPosition);
			double trailingEdge = tickerXPosition + halfWidth;
			
			// determine when I'm done
			if (!done && tickerXPosition < offLeft ) {
				//NSLog(@"setting outputDone to TRUE");
				self.outputDone = TRUE;
				done = TRUE;
				titleScrollingOut = TRUE;
				//NSLog(@"finished setting outputDone to TRUE");
				tickerScrolling = FALSE;
				// start our timer over so we can scroll out
				startTime = self.inputPatchTime;
			} else {
				self.outputDone = FALSE;
				done = FALSE;
			}
			
			// determine when child should start/refresh
			if (trailingEdge < 1 && trailingEdge > 0) {
				self.outputStartChild = TRUE;
			} else {
				self.outputStartChild = FALSE;
			}
			//NSLog(@"about to set outputTickerXPosition");
			self.outputTickerXPosition = tickerXPosition;
			
			////////////////////////
			// move the title in
			////////////////////////


			if (titleSlidingIn & runningTime <= SLIDEINDURATION ){
				
				
				halfWidth = TITLEWIDTH/2;
				offLeft = -1 - halfWidth;
				self.outputTitleBackgroundWidth = TITLEWIDTH;
				self.outputBackgroundHeight = TITLEBACKGROUNDHEIGHT;
				self.outputBorderWidth = BORDERWIDTH;
				
				titleXPosition = offLeft + TITLEWIDTH * runningTime/SLIDEINDURATION;
				
				self.outputTitleYPosition = BOTTOMY + TITLEBACKGROUNDHEIGHT/2;
				self.outputBorderXPosition = titleXPosition + halfWidth;
				self.outputTitleXPosition = titleXPosition;
				
			} else if (runningTime > SLIDEINDURATION) {
				self.outputTitleXPosition = -1 + TITLEWIDTH/2;
				titleSlidingIn = FALSE;
			}

			
		}
		if (titleScrollingOut ) {
			runningTime = self.inputPatchTime - startTime;
			if (runningTime <= SCROLLOUTDURATION ) {
				
				halfWidth = TITLEWIDTH/2;
				double halfHeight = TITLEBACKGROUNDHEIGHT/2;
				double onScreen = BOTTOMY + halfHeight;

				self.outputTitleBackgroundWidth = TITLEWIDTH;
				self.outputBackgroundHeight = TITLEBACKGROUNDHEIGHT;
				self.outputBorderWidth = BORDERWIDTH;
				
				double titleYPosition = onScreen - TITLEBACKGROUNDHEIGHT * runningTime/SCROLLOUTDURATION;
				
				self.outputTitleXPosition = -1 + halfWidth;
				self.outputBorderXPosition = -1 + TITLEWIDTH;
				self.outputTitleYPosition = titleYPosition;

				
			} else {
				self.outputTitleYPosition = -1 - TITLEBACKGROUNDHEIGHT/2;
				titleScrollingOut = FALSE;
			}

			
			
			
			
		}
		
	} else {
		// pass things through
		//NSLog(@"bypassing outputDone and outputStartChild");
		self.outputDone = self.inputParentDone;
		self.outputStartChild = self.inputParentSaysStart;
		//NSLog(@"finished bypassing outputDone and outputStartChild");

	}
	//NSLog(@"about to set and return");
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end
