//
//  VerifyConnectionPlugIn.m
//  VerifyConnection
//
//  Created by Adam Fenn on 4/8/11.
//  Copyright (c) 2011 Adam Fenn. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "VerifyConnectionPlugIn.h"
#import "AFMethods.h"
#import "WatchObject.h"

#define	kQCPlugIn_Name				@"Verify Connection"
#define	kQCPlugIn_Description		@"This patch will accept the basic configuration data for any Track Wrestling feed, and will validate that the configuration is correct, and will provide feedback to the user through various error conditions.  It will also provide a structure of valid matt/boutkey pairs, and the boutKey for the specificified mat.  It's important to know that it will refresh attempt to load the URL when the Refresh input transitions to TRUE, and it will process and output that data when the Refresh input transitions to FALSE.  The URL is given a two second timeout."

@implementation VerifyConnectionPlugIn
@synthesize oldRefresh, response, responseData, failWithError, newResults;
/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/
@dynamic inputServerName, inputLocalServer, inputMatName, inputTournamentId, inputRefresh, outputMatError, outputServerError, outputTournamentError, outputValidMatList, outputEventName, outputBoutKey, outputBaseUrl, outputTournamentType;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/

	
	if([key isEqualToString:@"inputServerName"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Server Name", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputLocalServer"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Local Server", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputMatName"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Mat Name", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"inputTournamentId"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Tournament ID", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"inputRefresh"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Refresh", QCPortAttributeNameKey,
						nil];		
	
	if([key isEqualToString:@"outputMatError"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Mat Error", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputServerError"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Server Error", QCPortAttributeNameKey,
						nil];	
	if([key isEqualToString:@"outputTournamentError"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"TournamentError", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputValidMatList"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Valid Mat List", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputEventName"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Event Name", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputBoutKey"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Bout Key", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputBaseUrl"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Base URL", QCPortAttributeNameKey,
						nil];
	if([key isEqualToString:@"outputTournamentType"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
						@"Type Index", QCPortAttributeNameKey,
						nil];

	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	[response release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[connection release];
	failWithError = TRUE;
	
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{
  response = [[NSString alloc] initWithData:responseData encoding: NSASCIIStringEncoding];
	[responseData release];
	[connection release];
	failWithError = FALSE;
	newResults = TRUE;

	
}

@end

@implementation VerifyConnectionPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/
	


	// don't do anything until the required fields are set
	if (self.inputServerName.length != 0) {
		
		// create the url and load it
		NSString *baseUrl = [AFMethods baseURLFromServerName:self.inputServerName localServer:self.inputLocalServer];

		NSString *nonBlankTournamentId = [NSString stringWithFormat:@"INVALID"];

				
		// identify the tournament type;
		if (self.inputTournamentId.length != 0 ) {
			
			// only look these up if tournament id is actually filled out
			NSArray *tournamentType  = [AFMethods tournamentTypeFromId:self.inputTournamentId];
			self.outputTournamentType = [tournamentType objectAtIndex:1];
			
			// because track won't show INVALID with a blank id, we pre-populated this field.  since there's something entered,
			// we need to put it back.
			
			nonBlankTournamentId = self.inputTournamentId;
			
			// set the base url to return
			self.outputBaseUrl = [NSString stringWithFormat:@"%@%@tournaments/", baseUrl, [tournamentType objectAtIndex:0]];	
			
		}
			
		// formulte the url string and pull the result
		NSString *urlString = [NSString stringWithFormat:@"%@VideoData.jsp?tournament=%@", baseUrl, nonBlankTournamentId];
	
	
		// start the URL load via async
		if (self.inputRefresh && self.inputRefresh != self.oldRefresh ) {
			NSURL *myURL = [NSURL URLWithString:urlString];
			NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:myURL
																														 cachePolicy:NSURLRequestReloadIgnoringLocalCacheData
																												 timeoutInterval:2];
			
			[[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:YES];
			
			// launch a new thread to update output ports when the reqeust is done
			[NSThread detachNewThreadSelector:@selector(updateWhenComplete:) toTarget:[WatchObject class] withObject:nil];
			self.newResults = FALSE;
		}
		self.oldRefresh = self.inputRefresh;

		
		if (self.newResults) {
			
			
			NSString *boutKey;
			NSString *urlResult;
			NSRange invalidRange;
			NSRange validRange;
			NSArray *returnedObjects;
			NSDictionary *validMatList;
			int i;
			NSString *mPrependedBoutKey;
			NSMutableArray *keys = [[NSMutableArray alloc] init];
			NSMutableArray *objects = [[NSMutableArray alloc] init];
			
			self.outputServerError = TRUE;
			self.outputTournamentError = TRUE;
			self.outputMatError = TRUE;
			self.outputEventName = [NSString stringWithString:@"Event Name"];
			
			urlResult = [NSString stringWithFormat:@"%@", self.response];
			
			urlResult = [urlResult stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			urlResult = [urlResult stringByTrimmingCharactersInSet:[NSCharacterSet newlineCharacterSet]];
			
			// determine if serverName, localServer and tournamentId are valid
			invalidRange = [urlResult rangeOfString:@"INVALID"];
			validRange = [urlResult rangeOfString:@"VALID"];
			
			if (invalidRange.location != NSNotFound && failWithError == FALSE) {
				self.outputServerError = FALSE;
			} else if (validRange.location != NSNotFound && failWithError == FALSE) {
				self.outputServerError = FALSE;
				self.outputTournamentError = FALSE;
				
				// build the matList
				returnedObjects = [urlResult componentsSeparatedByString: @"~"];
				if (returnedObjects.count >=3 ) {
					self.outputEventName = [returnedObjects objectAtIndex:1];
					
					for (i=3; i<=returnedObjects.count-1; i=i+2 ) {
						// need to redo this in a way that keeps the mats sorted
						mPrependedBoutKey = [NSString stringWithFormat:@"m%@", [returnedObjects objectAtIndex:i-1]];
						
						[keys addObject:[returnedObjects objectAtIndex:i]];					
						[objects addObject:mPrependedBoutKey];
					}
					// create the dictionary of valid mats from keys & objects arrays				
					validMatList = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
					
					self.outputValidMatList = [NSDictionary dictionaryWithDictionary:validMatList];
					
					if (self.inputMatName.length > 0) {
						boutKey = [validMatList valueForKey:self.inputMatName];
						// make sure the mat name is valid
						
						if (boutKey.length > 0) {
							self.outputMatError = FALSE;
						} else {
							boutKey = [NSString stringWithString:@""];
						}
						
						self.outputBoutKey = boutKey;
					}
				}
				
			} 
			[keys release];
			[objects release];
			self.newResults = FALSE;
			
		}

	}
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
